//+------------------------------------------------------------------+
//|                                        AdaptiveCyberCycle_v2.mq4 |
//|                             Copyright  2008-13, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4
// Price    = 7 - Heiken Ashi Close  
// Price    = 8 - Heiken Ashi Open
// Price    = 9 - Heiken Ashi High
// Price    =10 - Heiken Ashi Low


#property copyright "Copyright  2008-13, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  SteelBlue
#property indicator_width1  2
#property indicator_color2  Tomato
#property indicator_width2  1
#property indicator_level1  0


//---- input parameters
extern int     TimeFrame    =   0;  //Timeframe in min
extern int     Price        =   4;  //Applied to(see list above) 
extern double  Alpha        = 0.07; //Cycle Smoothing Factor
extern int     MedianPeriod =   5;  //Period of Moving Median
extern int     DCsmooth     =   5;  //Period of Dominant Cycle Smoothing
extern int     AdaptiveMode =   0;  //0-off,1-on
extern bool    LinesVisible = false;
extern string  LinesID      = "AdaptCyberCycle1";
extern color   LinesUpColor = LimeGreen;
extern color   LinesDnColor = OrangeRed;
extern int     LinesStyle   = STYLE_SOLID;
extern int     LinesWidth   = 0;
 
//---- indicator buffers
double AdaptCycle[];
double Trigger[];
double aPrice[];
double DeltaPhase[];
double cross[];

double   haClose[2], haOpen[2], haHigh[2], haLow[2];
double   Smooth[3], Cycle[7], Q1[2], I1[2], iPeriod[2], cPeriod[2];
datetime prevhatime, prevtime;
string   shortName, TF;
string   IndicatorName;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if(TimeFrame <= Period()) TimeFrame = Period();
   TF = tf();
   if(TF  == "Unknown timeframe") TimeFrame = Period();
//---- 
   IndicatorBuffers(5);
   SetIndexBuffer(0,AdaptCycle); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1,   Trigger); SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(2,    aPrice);   
   SetIndexBuffer(3,DeltaPhase);
   SetIndexBuffer(4,cross);
   IndicatorName = WindowExpertName();  
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)); 
   shortName = LinesID+" ["+TF+"]("+Price+","+DoubleToStr(Alpha,3)+")";
   IndicatorShortName(shortName);
   SetIndexLabel(0,"CyberCycle");
   SetIndexLabel(1,"Trigger");
//----
   SetIndexDrawBegin(0,7);
   SetIndexDrawBegin(1,7);
   
   return(0);
}
int deinit()
{
   string find = LinesID+":";
   for (int i=ObjectsTotal()-1; i>= 0; i--)
   {
      string name = ObjectName(i); if (StringFind(name,find)==0) ObjectDelete(name);
   }
return(0); 
}

//+------------------------------------------------------------------+
//| AdaptiveCyberCycle_v2                                            |
//+------------------------------------------------------------------+
int start()
{
   int i, shift, limit, counted_bars=IndicatorCounted();
   
   if(counted_bars < 0) return(0);  
   if(counted_bars > 0) limit  = MathMin(Bars-counted_bars,Bars-1);
   if(counted_bars < 1) 
   {
   limit=Bars-1; 
   for(i=0;i<limit;i++) {AdaptCycle[i] = 0; Trigger[i] = 0;}
   }
//----
   int window = WindowFind(shortName);
   if(TimeFrame != Period())
	{
   limit = MathMax(limit,TimeFrame/Period());   
      
      for(shift = 0;shift < limit;shift++) 
      {	
      int y = iBarShift(NULL,TimeFrame,Time[shift]);
      
      AdaptCycle[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Alpha,MedianPeriod,DCsmooth,AdaptiveMode,LinesVisible,LinesID,LinesUpColor,LinesDnColor,LinesStyle,LinesWidth,shortName,0,y);
      Trigger[shift]    = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Alpha,MedianPeriod,DCsmooth,AdaptiveMode,LinesVisible,LinesID,LinesUpColor,LinesDnColor,LinesStyle,LinesWidth,shortName,1,y);
      cross[shift]      = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Alpha,MedianPeriod,DCsmooth,AdaptiveMode,LinesVisible,LinesID,LinesUpColor,LinesDnColor,LinesStyle,LinesWidth,shortName,4,y);
      }  
	
	return(0);
	}
	else 
   for(shift=limit;shift>=0;shift--)
   {	
      if(prevtime != Time[shift])
      {
      for(i=6;i>=1;i--) Cycle[i] = Cycle[i-1];
      Smooth[2] = Smooth[1];
      Smooth[1] = Smooth[0];
      I1[1] = I1[0];
      Q1[1] = Q1[0];
      iPeriod[1] = iPeriod[0];
      cPeriod[1] = cPeriod[0];
      prevtime = Time[shift];
      }   

      if(Price <= 6) aPrice[shift] = iMA(NULL,0,1,0,0,Price,shift);   
      else
      if(Price > 6 && Price <= 10) aPrice[shift] = HeikenAshi(Price-7,shift);   
      
   if(shift < Bars - 3) Smooth[0] = (aPrice[shift] + 2*aPrice[shift+1] + 2*aPrice[shift+2] + aPrice[shift+3])/6; 
	Cycle[0] = (1 - 0.5*Alpha)*(1 - 0.5*Alpha)*(Smooth[0] - 2*Smooth[1] + Smooth[2]) + 2*(1-Alpha)*Cycle[1] - (1-Alpha)*(1-Alpha)*Cycle[2];
	if(shift > Bars - 7) Cycle[0] = (aPrice[shift] - 2*aPrice[shift+1] + aPrice[shift+2])/4;  
	  
	   if(AdaptiveMode == 0) AdaptCycle[shift] = Cycle[0];
	   else
      {
      Q1[0] = (0.0962*Cycle[0] + 0.5769*Cycle[2] - 0.5769*Cycle[4] - 0.0962*Cycle[6])*(0.5+0.08*iPeriod[1]);
      I1[0] = Cycle[3];  
	
      if(Q1[0] != 0 && Q1[1] != 0) DeltaPhase[shift] = (I1[0]/Q1[0] - I1[1]/Q1[1])/(1 + I1[0]*I1[1]/(Q1[0]*Q1[1]));		
      if(DeltaPhase[shift] < 0.1) DeltaPhase[shift] = 0.1;  
      if(DeltaPhase[shift] > 1.1) DeltaPhase[shift] = 1.1;
      double MedianDelta = Median(DeltaPhase,MedianPeriod,shift);

      if(MedianDelta == 0) double DC = 15; else DC = 6.28318/MedianDelta + 0.5;
	
      iPeriod[0] = 0.33*DC + (1 - 2.0/(DCsmooth + 1))*iPeriod[1];
      cPeriod[0] = 0.15*iPeriod[0] + 0.85*cPeriod[1];
   
      double alpha1 = 2.0/(cPeriod[0] + 1);
   
   
      if(shift > Bars - 7) AdaptCycle[shift] = Cycle[0];
      else 
      AdaptCycle[shift] = (1 - 0.5*alpha1)*(1 - 0.5*alpha1)*(Smooth[0] - 2*Smooth[1] + Smooth[2]) + 2*(1 - alpha1)*AdaptCycle[shift+1] - (1 - alpha1)*(1 - alpha1)*AdaptCycle[shift+2];
      }
     
   Trigger[shift] = AdaptCycle[shift+1];
   cross[shift] = cross[i+1];
   if (AdaptCycle[shift]>Trigger[shift]) cross[shift] = 1;
   if (AdaptCycle[shift]<Trigger[shift]) cross[shift] =-1;
   if (LinesVisible && window>-1)
 	{
 	   string name = LinesID+":"+Time[shift];
 	      ObjectDelete(name);
 	      if (cross[shift]!=cross[shift+1])
 	      {
 	         color theColor  = LinesUpColor; if (cross[shift]==-1) theColor = LinesDnColor;
 	            ObjectCreate(name,OBJ_VLINE,window,Time[shift],0);
 	               ObjectSet(name,OBJPROP_WIDTH,LinesWidth);
 	               ObjectSet(name,OBJPROP_STYLE,LinesStyle);
 	               ObjectSet(name,OBJPROP_COLOR,theColor);
 	      }
 	}
   
}
return(0);	
}


//----
double Median(double price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar+i];
   ArraySort(array);
   
   int num = MathRound(0.5*(per-1)); 
   if(MathMod(per,2)>0) double median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}

// HeikenAshi Price:  7 - Close,8 - Open,9 - High,10 - Low 
double HeikenAshi(int price,int bar)
{ 
   if(prevhatime != Time[bar])
   {
   haClose[1] = haClose[0];
   haOpen [1] = haOpen [0];
   haHigh [1] = haHigh [0];
   haLow  [1] = haLow  [0];
   prevhatime = Time[bar];
   }
   
   if(bar == Bars - 1) 
   {
   haClose[0] = Close[bar];
   haOpen [0] = Open [bar];
   haHigh [0] = High [bar];
   haLow  [0] = Low  [bar];
   }
   else
   {
   haClose[0] = (Open[bar] + High[bar] + Low[bar] + Close[bar])/4;
   haOpen [0] = (haOpen[1] + haClose[1])/2;
   haHigh [0] = MathMax(High[bar],MathMax(haOpen[0],haClose[0]));
   haLow  [0] = MathMin(Low [bar],MathMin(haOpen[0],haClose[0]));
   }
   
   switch(price)
   {
   case 0: return(haClose[0]); break;
   case 1: return(haOpen [0]); break;
   case 2: return(haHigh [0]); break;
   case 3: return(haLow  [0]); break;
   }
} 

string tf()
{
   switch(TimeFrame)
   {
   case PERIOD_M1:   return("M1");
   case PERIOD_M5:   return("M5");
   case PERIOD_M15:  return("M15");
   case PERIOD_M30:  return("M30");
   case PERIOD_H1:   return("H1");
   case PERIOD_H4:   return("H4");
   case PERIOD_D1:   return("D1");
   case PERIOD_W1:   return("W1");
   case PERIOD_MN1:  return("MN1");
   default:          return("Unknown timeframe");
   }
}	                		